#!/bin/sh
#
# Nagios plugin to check network interfaces on Solaris.
#
# $Id: check_solaris_nics.sh,v 1.11 2008/08/27 08:09:12 kivimaki Exp $
#
# Copyright (C) 2007-2008  Hannu Kivimki / CSC - Scientific Computing Ltd.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

# External commands
CMD_AWK="/usr/bin/awk"
CMD_KSTAT="/usr/bin/kstat"
CMD_NDD="/usr/sbin/ndd"
CMD_SED="/usr/bin/sed"
CMD_WHOAMI="/usr/ucb/whoami"

# SETUID MOD:
# If plugin is ran as non-root user, duplex setting for some interfaces
# cannot be determined as the drivers do not provide this information
# in kstat structures. With drivers like hme and e1000g (older versions)
# the only way to check duplex is to use ndd (or dladm on Solaris 10), which
# requires root priviledges. If you need to check hme interfaces as non-root,
# compile check_solaris_nics_nddgetdx.c setuid wrapper (see source for more
# information) and uncomment and edit (fix path) the following:
#CMD_NDDGETDX="/path/to/nagios/libexec/check_solaris_nics_nddgetdx"

# Nagios plugin return values
STATE_OK=0
STATE_WARNING=1
STATE_CRITICAL=2
STATE_UNKNOWN=3
STATE_TEXT=""
STATE_TEXT_FAULTY=""

# Default values
IF_COUNT=0
FAULTY_IF_COUNT=0
SPEED_ERRORS=0
DUPLEX_ERRORS=0
LINK_ERRORS=0
ERRORS_ONLY=0
VERBOSE=0


# ------------------------------ FUNCTIONS -------------------------------------

printInfo() {
    echo "Nagios plugin to check network interfaces on Solaris."
    echo "Copyright (C) 2007-2008  Hannu Kivimki / CSC - Scientific Computing Ltd."
}

printHelp() {
    echo
    echo "Usage: check_solaris_nics.sh [-e] -i <interface> -s <speed> -d <duplex> [-i <interface2> ...]"
    echo
    echo "  -e  errors only - if something is wrong, show just errorneous interfaces, not all"
    echo "  -i  interface to check (bgeN | ceN | e1000gN | eriN | hmeN | nxgeN | qfeN, N=0..9)"
    echo "  -s  expected link speed in Mbps (10 | 100 | 1000)"
    echo "  -d  expected link duplex (half | full)"
    echo "  -h  this help screen"
    echo "  -l  license info"
    echo "  -v  verbose output (for debugging)"
    echo "  -V  version info"
    echo
    echo "Example: check_solaris_nics.sh -i ce0 -s 1000 -d full -i ce1 -s 10 -d half"
    echo
    echo "Return values:"
    echo
    echo "  0 (OK)       interface(s) up with correct speed and duplex"
    echo "  1 (WARNING)  interface(s) up but incorrect speed or duplex"
    echo "  2 (CRITICAL) one or more interfaces down"
    echo "  3 (UNKNOWN)  plugin error"
    echo
}

printLicense() {
    echo
    echo "This program is free software; you can redistribute it and/or"
    echo "modify it under the terms of the GNU General Public License"
    echo "as published by the Free Software Foundation; either version 2"
    echo "of the License, or (at your option) any later version."
    echo
    echo "This program is distributed in the hope that it will be useful,"
    echo "but WITHOUT ANY WARRANTY; without even the implied warranty of"
    echo "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the"
    echo "GNU General Public License for more details."
    echo
    echo "You should have received a copy of the GNU General Public License"
    echo "along with this program; if not, write to the Free Software"
    echo "Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA."
    echo
}

printVersion() {
    echo
    echo "\$Id: check_solaris_nics.sh,v 1.11 2008/08/27 08:09:12 kivimaki Exp $"
    echo
}


# Checks command line options (pass $@ as parameter) and stores values to
# IF_[1..n], MODULE_[1..N], INSTANCE_[1..N], SPEED_[1..n] and DUPLEX_[1..n].
#
# Example: IF_1=ce0 MODULE_1=ce INSTANCE_1=0 SPEED_1=1000 DUPLEX_1=full
#          IF_2=ce1 MODULE_2=ce INSTANCE_2=1 SPEED_2=10   DUPLEX_2=half
#
checkOptions() {
    while getopts ei:s:d:lhvV OPT $@; do
        case $OPT in
            e) ERRORS_ONLY=1
               ;;
            i) IF_COUNT=`expr $IF_COUNT + 1`
               eval IF_${IF_COUNT}=$OPTARG
               eval MODULE_${IF_COUNT}=`echo $OPTARG | $CMD_SED 's/[0-9]*$//'`
               eval INSTANCE_${IF_COUNT}=`echo $OPTARG | $CMD_SED 's/.*[a-z]\([0-9]*\)$/\1/'`
               ;;
            s) eval SPEED_${IF_COUNT}=$OPTARG
               ;;
            d) eval DUPLEX_${IF_COUNT}=$OPTARG
               ;;
            l) printInfo
               printLicense
               exit $STATE_UNKNOWN
               ;;
            h) printInfo
               printHelp
               exit $STATE_UNKNOWN
               ;;
            v) VERBOSE=1
               ;;
            V) printInfo
               printVersion
               exit $STATE_UNKNOWN
               ;;
            ?) printInfo
               printHelp
               exit $STATE_UNKNOWN
               ;;
        esac
    done

    if [ $IF_COUNT -eq 0 ]; then
        printInfo
        printHelp
        exit $STATE_UNKNOWN
    else
        # Validate driver, instance, speed and duplex values for all interfaces.
        #
        i=1
        while [ $i -le $IF_COUNT ]; do
            eval MODULE=\$MODULE_$i
            case $MODULE in
                bge | ce | e1000g | eri | hme | nxge | qfe) ;;
                *) echo "Error: Unsupported network interface driver ($MODULE)."
                   printHelp
                   exit $STATE_UNKNOWN
                   ;;
            esac
            eval INSTANCE=\$INSTANCE_$i
            case $INSTANCE in
                0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9) ;;
                *) echo "Error: Missing or incorrect interface instance number N ($INSTANCE)."
                   printHelp
                   exit $STATE_UNKNOWN
                   ;;
            esac
            eval SPEED=\$SPEED_$i
            case $SPEED in
                10 | 100 | 1000) ;;
                *) eval echo "Error: Missing or incorrect speed for interface \$IF_$i."
                   printHelp
                   exit $STATE_UNKNOWN
                   ;;
            esac
            eval DUPLEX=\$DUPLEX_$i
            case $DUPLEX in
                half | full) ;;
                *) eval echo "Error: Missing or incorrect duplex for interface \$IF_$i."
                   printHelp
                   exit $STATE_UNKNOWN
                   ;;
            esac
            i=`expr $i + 1`
        done
    fi
}


# ----------------------------- MAIN PROGRAM -----------------------------------

checkOptions $@

if [ "`$CMD_WHOAMI`" = "root" ]; then
    RUNNING_AS_ROOT=1
    if [ $VERBOSE -eq 1 ]; then
        echo "Running script with root priviledges (can use ndd) ..."
    fi
fi

i=1
while [ $i -le $IF_COUNT ]; do
    eval IF=\$IF_$i
    eval MODULE=\$MODULE_$i
    eval INSTANCE=\$INSTANCE_$i
    eval EXP_SPEED=\$SPEED_$i
    eval EXP_DUPLEX=\$DUPLEX_$i
    IF_FAULTY=0

    if [ $VERBOSE -eq 1 ]; then
        echo "Checking interface $IF (expecting $EXP_SPEED Mbps $EXP_DUPLEX duplex) ..."
    fi

    # Check link status (up/down) with kstat.
    #
    # Different bge driver versions have different kstat structures:
    # bge:0:parameters:link_status
    # bge0:0:bge0/0:link_up        (NOTE: bge1:0:bge1/0)
    #
    # Different e1000g driver versions have different kstat structures:
    # e1000g:0:statistics:link_up
    # e1000g0:0:e1000g0/0:link_up  (NOTE: e1000g1:0:e1000g1/0)
    # e1000g:0:mac:link_up
    #
    # Different nxge driver versions have different kstat structures:
    # nxge0:0:nxge0/0:link_up      (NOTE: nxge1:0:nxge1/0)
    # nxge:0:Port Stats:link_up
    #
    # ce:0:ce0:link_up
    # eri:0:eri0:link_up
    # hme:0:hme0:link_up
    # qfe:0:qfe0:link_up
    #
    case $MODULE in
        bge)
            # New kstat structure.
            KSTAT_REFERENCE="${MODULE}$INSTANCE:0:${MODULE}$INSTANCE/0:link_up"
            LINK_UP=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/link_up/ {print \$2}'`
            if [ "$LINK_UP" = "" ]; then
                # Old kstat structure
                KSTAT_REFERENCE="$MODULE:$INSTANCE:parameters:link_status"
                LINK_UP=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/link_status/ {print \$2}'`
            fi
            ;;
        e1000g)
            # New kstat structure.
            KSTAT_REFERENCE="$MODULE:$INSTANCE:mac:link_up"
            LINK_UP=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/link_up/ {print \$2}'`
            if [ "$LINK_UP" = "" ]; then
                # Old kstat structure 1.
                KSTAT_REFERENCE="${MODULE}$INSTANCE:0:${MODULE}$INSTANCE/0:link_up"
                LINK_UP=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/link_up/ {print \$2}'`
            fi
            if [ "$LINK_UP" = "" ]; then
                # Old kstat structure 2.
                KSTAT_REFERENCE="$MODULE:$INSTANCE:statistics:link_up"
                LINK_UP=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/link_up/ {print \$2}'`
            fi
            ;;
        nxge)
            # New kstat structure.
            KSTAT_REFERENCE="$MODULE:$INSTANCE:Port Stats:link_up"
            LINK_UP=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/link_up/ {print \$2}'`
            if [ "$LINK_UP" = "" ]; then
                # Old kstat structure.
                KSTAT_REFERENCE="${MODULE}$INSTANCE:0:${MODULE}$INSTANCE/0:link_up"
                LINK_UP=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/link_up/ {print \$2}'`
            fi
            ;;
        *)
            KSTAT_REFERENCE="$MODULE:$INSTANCE:$IF:link_up"
            LINK_UP=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/link_up/ {print \$2}'`
            ;;
    esac

    # If link is up, check speed and duplex with kstat or ndd
    # depending on the driver module.
    #
    if [ "$LINK_UP" -eq 1 ]; then
        case $MODULE in
            bge) # kstat
                 # Different bge driver versions have different kstat structures:
                 # bge:0:parameters:link_speed  10 | 100 | 1000
                 # bge:0:parameters:link_duplex  0 | 1 | 2 (down, half, full)
                 #
                 # bge0:0:bge0/0:ifspeed 10000000 | 100000000 | 1000000000
                 # bge0:0:bge0/0:link_duplex  0 | 1 | 2 (down, half, full)
                 #
                 KSTAT_REFERENCE="${MODULE}$INSTANCE:0:${MODULE}$INSTANCE/0:ifspeed"
                 REAL_SPEED=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/ifspeed/ {print \$2}'`
                 if [ "$REAL_SPEED" != "" ]; then
                    # New kstat structure.
                    KSTAT_REFERENCE="${MODULE}$INSTANCE:0:${MODULE}$INSTANCE/0:link_duplex"
                    REAL_DUPLEX=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/link_duplex/ {print \$2}'`
                 else
                    # Old kstat structure.
                    KSTAT_REFERENCE="$MODULE:$INSTANCE:parameters:link_speed"
                    REAL_SPEED=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/link_speed/ {print \$2}'`
                    KSTAT_REFERENCE="$MODULE:$INSTANCE:parameters:link_duplex"
                    REAL_DUPLEX=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/link_duplex/ {print \$2}'`
                 fi
                 ;;
            ce)  # kstat
                 # ce:0:ce0:link_speed  10 | 100 | 1000
                 # ce:0:ce0:link_duplex  0 | 1 | 2 (down, half, full)
                 #
                 KSTAT_REFERENCE="$MODULE:$INSTANCE:$IF:link_speed"
                 REAL_SPEED=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/link_speed/ {print \$2}'`
                 KSTAT_REFERENCE="$MODULE:$INSTANCE:$IF:link_duplex"
                 REAL_DUPLEX=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/link_duplex/ {print \$2}'`
                 ;;
            e1000g) # kstat
                 # Different e1000g driver versions have different kstat structures:
                 # e1000g:0:statistics:link_speed  10 | 100 | 1000
                 # (no duplex information available)
                 #
                 # e1000g1:0:e1000g1/0:ifspeed 10000000 | 100000000 | 1000000000
                 # e1000g1:0:e1000g1/0:link_duplex  0 | 1 | 2 (down, half, full)
                 #
                 # e1000g:0:mac:ifspeed  10000000 | 100000000 | 1000000000
                 # e1000g:0:mac:link_duplex  0 | 1 | 2 (down, half, full)
                 #
                 KSTAT_REFERENCE="$MODULE:$INSTANCE:mac:ifspeed"
                 REAL_SPEED=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/ifspeed/ {print \$2}'`
                 if [ "$REAL_SPEED" != "" ]; then
                    # New kstat structure.
                    KSTAT_REFERENCE="$MODULE:$INSTANCE:mac:link_duplex"
                    REAL_DUPLEX=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/link_duplex/ {print \$2}'`
                 else
                    # Old kstat structure 1.
                    KSTAT_REFERENCE="${MODULE}$INSTANCE:0:${MODULE}$INSTANCE/0:ifspeed"
                    REAL_SPEED=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/ifspeed/ {print \$2}'`
                    if [ "$REAL_SPEED" != "" ]; then
                        KSTAT_REFERENCE="${MODULE}$INSTANCE:0:${MODULE}$INSTANCE/0:link_duplex"
                        REAL_DUPLEX=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/link_duplex/ {print \$2}'`
                    else
                        # Old kstat structure 2.
                        KSTAT_REFERENCE="$MODULE:$INSTANCE:statistics:link_speed"
                        REAL_SPEED=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/link_speed/ {print \$2}'`
                        #
                        # Unless SETUID MOD is used, try to get e1000g duplex with
                        # ndd (will only succeed if plugin is ran as root).
                        #
                        # ndd -get /dev/e1000g0 link_duplex   0 | 1 | 2 (down, half, full)
                        # check_solaris_nics_nddgetdx e1000g 0   0 | 1 | 2 (down, half, full)
                        #
                        if [ $CMD_NDDGETDX ]; then
                           REAL_DUPLEX=`$CMD_NDDGETDX $MODULE $INSTANCE`
                        elif [ $RUNNING_AS_ROOT ]; then
                           REAL_DUPLEX=`$CMD_NDD -get /dev/$MODULE$INSTANCE link_duplex`
                        else
                           REAL_DUPLEX="unknown"
                        fi
                    fi
                 fi
                 ;;
            eri | qfe) # kstat
                 # eri:0:eri0:ifspeed  10000000 | 100000000 | 1000000000
                 # eri:0:eri0:link_duplex  0 | 1 | 2 (down, half, full)
                 #
                 KSTAT_REFERENCE="$MODULE:$INSTANCE:$IF:ifspeed"
                 REAL_SPEED=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/ifspeed/ {print \$2}'`
                 KSTAT_REFERENCE="$MODULE:$INSTANCE:$IF:link_duplex"
                 REAL_DUPLEX=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/link_duplex/ {print \$2}'`
                 ;;
            hme) # kstat
                 # hme:0:hme0:ifspeed  10000000 | 100000000 | 1000000000
                 # hme:0:hme0:link_duplex  not usable, always zero?
                 #
                 KSTAT_REFERENCE="$MODULE:$INSTANCE:$IF:ifspeed"
                 REAL_SPEED=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/ifspeed/ {print \$2}'`

                 # Unless SETUID MOD is used, try to get hme duplex with
                 # ndd (will only succeed if plugin is ran as root).
                 #
                 # ndd -set /dev/hme instance 0
                 # ndd -get /dev/hme link_mode  0 | 1 (half, full)
                 # check_solaris_nics_nddgetdx hme 0   0 | 1 (half, full)
                 #
                 if [ $CMD_NDDGETDX ]; then
                    REAL_DUPLEX=`$CMD_NDDGETDX $MODULE $INSTANCE`
                    # Add + 1 because we wan't to have either value of 1 for
                    # half duplex or 2 for full duplex like other drivers:
                    REAL_DUPLEX=`expr $REAL_DUPLEX + 1`
                 elif [ $RUNNING_AS_ROOT ]; then
                    $CMD_NDD -set /dev/$MODULE instance 0 > /dev/null 2>&1
                    REAL_DUPLEX=`$CMD_NDD -get /dev/$MODULE link_mode`
                    REAL_DUPLEX=`expr $REAL_DUPLEX + 1`
                 else
                    REAL_DUPLEX="unknown"
                 fi
                 ;;
            nxge) # kstat
                 # Different nxge driver versions have different kstat structures:
                 # nxge0:0:nxge0/0:ifspeed 10000000 | 100000000 | 1000000000
                 # nxge0:0:nxge0/0:link_duplex  0 | 1 | 2 (down, half, full)
                 #
                 # nxge:0:Port Stats:ifspeed 10000000 | 100000000 | 1000000000
                 # nxge:0:Port Stats:link_duplex  unknown | half | full
                 #
                 KSTAT_REFERENCE="${MODULE}:$INSTANCE:Port Stats:ifspeed"
                 REAL_SPEED=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/ifspeed/ {print \$2}'`
                 if [ "$REAL_SPEED" != "" ]; then
                    # New kstat structure.
                    KSTAT_REFERENCE="${MODULE}:$INSTANCE:Port Stats:link_duplex"
                    REAL_DUPLEX=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/link_duplex/ {print \$2}'`
                 else
                    # Old kstat structure.
                    KSTAT_REFERENCE="${MODULE}$INSTANCE:0:${MODULE}$INSTANCE/0:ifspeed"
                    REAL_SPEED=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/ifspeed/ {print \$2}'`
                    KSTAT_REFERENCE="${MODULE}$INSTANCE:0:${MODULE}$INSTANCE/0:link_duplex"
                    REAL_DUPLEX=`$CMD_KSTAT "$KSTAT_REFERENCE" | $CMD_AWK '/link_duplex/ {print \$2}'`
                 fi
                 ;;
            *) echo "Oops. This is a bug - fix network driver validation in checkOptions()."
               exit $STATE_WARNING
               ;;
        esac

        # Convert different speed and duplex styles.
        #
        case $REAL_SPEED in
            10 | 10000000 ) REAL_SPEED=10
                   ;;
            100 | 100000000 ) REAL_SPEED=100
                   ;;
            1000 | 1000000000 ) REAL_SPEED=1000
                   ;;
               * ) REAL_SPEED="unknown"
                   ;;
        esac
        case $REAL_DUPLEX in
            1 | half ) REAL_DUPLEX="half"
                       ;;
            2 | full ) REAL_DUPLEX="full"
                       ;;
               * ) REAL_DUPLEX="unknown"
                   ;;
        esac

        # Set error flags if speed or duplex is someting else than expected.
        #
        if [ $REAL_SPEED -ne $EXP_SPEED ]; then
            SPEED_ERRORS=1
            IF_FAULTY=1
        fi
        if [ "$REAL_DUPLEX" != "$EXP_DUPLEX" ]; then
            DUPLEX_ERRORS=1
            IF_FAULTY=1
        fi

        eval IF_STATUS_$i=\"\$IF_$i up \$REAL_SPEED Mbps \$REAL_DUPLEX duplex\"

    else
        # Link was down, no point to check speed or duplex.
        #
        LINK_ERRORS=1
        IF_FAULTY=1
        eval IF_STATUS_$i=\"\$IF_$i down\"
    fi

    if [ $VERBOSE -eq 1 ]; then
        eval echo \$IF_STATUS_$i
    fi

    # Add interface status to plugin output.
    #
    eval STATE_TEXT=\"\$STATE_TEXT\$IF_STATUS_$i\"
    if [ $i -lt $IF_COUNT ]; then
        STATE_TEXT="$STATE_TEXT; "
    fi

    if [ $IF_FAULTY -eq 1 ]; then
        FAULTY_IF_COUNT=`expr $FAULTY_IF_COUNT + 1`
        if [ $FAULTY_IF_COUNT -gt 1 ]; then
            STATE_TEXT_FAULTY="$STATE_TEXT_FAULTY; "
        fi
        eval STATE_TEXT_FAULTY=\"\$STATE_TEXT_FAULTY\$IF_STATUS_$i\"
    fi

    i=`expr $i + 1`
done

# All done, print interface status info and exit.
#
if [ $ERRORS_ONLY -eq 1 ] && [ $FAULTY_IF_COUNT -gt 0 ]; then
    STATE_TEXT=$STATE_TEXT_FAULTY
fi
if [ $LINK_ERRORS -gt 0 ]; then
    echo "CRITICAL - $STATE_TEXT"
    exit $STATE_CRITICAL
elif [ $SPEED_ERRORS -gt 0 ]; then
    echo "SPEED WARNING - $STATE_TEXT"
    exit $STATE_WARNING
elif [ $DUPLEX_ERRORS -gt 0 ]; then
    echo "DUPLEX WARNING - $STATE_TEXT"
    exit $STATE_WARNING
fi

echo "OK - $STATE_TEXT"
exit $STATE_OK
